/*******************************************************************
 *
 *	File:		GameApp.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <eikenv.h>
#include <eikdialg.h>
#include <eiktxlbx.h>
#include <eiktxlbm.h>
#include <eikdlgtb.h>
#include <eiklabel.h>
#include <eikchlst.h>
#include <eikchkbx.h>
#ifndef __ER6__
#include <eikdialg.hrh>
#endif

#include "GameApp.h"
#include "Doomapp.rsg"
#include "version.h"
#include "AppPlatform.h"


/************************************************
 *
 * GameRunning Dialog
 *
 ************************************************/

class CGameRunningDialog : public CEikDialog
{
public:
	CGameRunningDialog(const TDesC8& aGameName, CGameEngine& aGameEngine);
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
	void HandleControlStateChangeL(TInt aControlId);

	TBuf<64>			iGameName;
	CGameEngine&		iGameEngine;
};

CGameRunningDialog::CGameRunningDialog(const TDesC8& aGameName, CGameEngine& aGameEngine) : iGameEngine(aGameEngine)
{
	iGameName.Copy(aGameName);
}

void CGameRunningDialog::PreLayoutDynInitL()
{
	CEikLabel*	label =(CEikLabel*)Control(ECtlRunningGameName);
	label->SetTextL(iGameName);
}

void CGameRunningDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch (aControlId)
	{
		case ECtlCmdButPanelRunningGame:
		{
			const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, Control(aControlId))->StateChangeControlId();
			switch (ctrlId)
			{
				case ECtlCmdButStopGame:
					iGameEngine.ForceGameExit();
					break;
				default:
					break;
			}
			break;
		}
		default:
			break;
	}
}

void CGameRunningDialog::SetSizeAndPositionL(const TSize& aSize)
{
	SET_CORNER_AND_SIZE(EHCenterVCenter, aSize);
}


/************************************************
 *
 * About Dialog
 *
 ************************************************/


class CAboutDialog : public CEikDialog
{
public:
	CAboutDialog(){}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
};

void CAboutDialog::PreLayoutDynInitL()
{
	TBuf<128>	text;
	TBuf<16>	versionText;

#if (EMAME_VERSION_PATCH > 0)
	versionText.Format(_L("%d.%d.%d"), EMAME_VERSION_MAJOR, EMAME_VERSION_MINOR, EMAME_VERSION_PATCH);
#else
	versionText.Format(_L("%d.%d"), EMAME_VERSION_MAJOR, EMAME_VERSION_MINOR);
#endif

	HBufC*		versionFormat =	CEikonEnv::Static()->AllocReadResourceL(R_ABOUT_VERSION_TEXT);
	CleanupStack::PushL(versionFormat);

	const TDesC&	versionFormatDes = versionFormat->Des();
	text.Format(versionFormatDes, &versionText);

	CEikLabel*	label =(CEikLabel*)Control(ECtlAboutVersion);
	label->SetTextL(text);

	CleanupStack::PopAndDestroy(); // 	versionFormat;
}

void CAboutDialog::SetSizeAndPositionL(const TSize& aSize)
{
//	TSize	size(400, 200);
	SET_CORNER_AND_SIZE(EHCenterVCenter, aSize);
}



/************************************************
 *
 * Help Dialog
 *
 ************************************************/

class CGameHelpDialog : public CEikDialog
{
public:
	CGameHelpDialog(){}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
};

void CGameHelpDialog::PreLayoutDynInitL()
{
}

void CGameHelpDialog::SetSizeAndPositionL(const TSize& aSize)
{
//	TSize	size(400, 200);
	SET_CORNER_AND_SIZE(EHCenterVCenter, aSize);
}



/************************************************
 *
 * Game Options Dialog
 *
 ************************************************/


class CGameOptionsDialog : public CEikDialog
{
public:
	CGameOptionsDialog(TGameOptions& aGameOptions, TBool aIsColorDevice) : iGameOptions(aGameOptions), iIsColorDevice(aIsColorDevice){}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
    TBool OkToExitL(TInt aKeycode);
	TGameOptions&		iGameOptions;
	TBool				iIsColorDevice;
};


void CGameOptionsDialog::PreLayoutDynInitL()
{
	CEikChoiceList* choiceList;

	CEikCheckBox* checkBox=(CEikCheckBox*)Control(ECtlGameOptionsSound);
	checkBox->SetState(iGameOptions.iSoundEnabled ? CEikButtonBase::ESet : CEikButtonBase::EClear);

//	if (iIsColorDevice)
	if (ETrue)
	{
		choiceList=(CEikChoiceList*)Control(ECtlGameOptionsGameSize);
		choiceList->SetCurrentItem((TInt) iGameOptions.iGameSize);

		DeleteLine(ECtlGameOptionsOrientation);
		DeleteLine(ECtlGameOptionsContrast);
		DeleteLine(ECtlGameOptionsSound);
	}
	else
	{
		choiceList=(CEikChoiceList*)Control(ECtlGameOptionsOrientation);
		choiceList->SetCurrentItem((TInt) iGameOptions.iOrientation);

		choiceList=(CEikChoiceList*)Control(ECtlGameOptionsContrast);
		choiceList->SetCurrentItem((TInt) iGameOptions.iContrast);

		DeleteLine(ECtlGameOptionsGameSize);
	}
}



void CGameOptionsDialog::SetSizeAndPositionL(const TSize& aSize)
{
	TSize size = aSize;
	if (size.iWidth < 400)
		size.iWidth = 400;
	SET_CORNER_AND_SIZE(EHCenterVCenter, size);
}

    
TBool CGameOptionsDialog::OkToExitL(TInt /*aKeycode*/)
{
	CEikCheckBox* checkBox;
	CEikChoiceList* choiceList;

//	if (iIsColorDevice)
	if (ETrue)
	{
		choiceList=(CEikChoiceList*)Control(ECtlGameOptionsGameSize);
		iGameOptions.iGameSize = (TGameOptions::TGameSize) choiceList->CurrentItem();
	}
	else
	{
		CEikCheckBox* checkBox=(CEikCheckBox*)Control(ECtlGameOptionsSound);
		iGameOptions.iSoundEnabled = (checkBox->State() == CEikButtonBase::ESet) ? (TBool) ETrue : (TBool) EFalse;

		choiceList=(CEikChoiceList*)Control(ECtlGameOptionsOrientation);
		iGameOptions.iOrientation = (TGameOptions::TOrientation) choiceList->CurrentItem();

		choiceList=(CEikChoiceList*)Control(ECtlGameOptionsContrast);
		iGameOptions.iContrast = (TGameOptions::TContrast) choiceList->CurrentItem();
	}

	return ETrue;
}


/************************************************
 *
 * Play List Dialog
 *
 ************************************************/

class CPlayListDialog : public CEikDialog
{
public:
	CPlayListDialog(CGameEngine& aGameEngine, TGameName& aGameToRun) : iGameEngine(aGameEngine), iGameToRun(aGameToRun){}
protected: // framework
    void PreLayoutDynInitL();
	void HandleControlStateChangeL(TInt aControlId);
	void SetSizeAndPositionL(const TSize& aSize);
	void ShowGameDetailsL(TInt aGame);
	void GetGameName();

    TBool OkToExitL(TInt aKeycode);
	CGameEngine&		iGameEngine;
	TGameName&			iGameToRun;
};

void CPlayListDialog::SetSizeAndPositionL(const TSize& aSize)
{
	TSize dialogSize(400, aSize.iHeight);
	SET_CORNER_AND_SIZE(EHCenterVCenter,dialogSize);
}


void CPlayListDialog::ShowGameDetailsL(TInt /*aGame*/)
{
}

void CPlayListDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch(aControlId)
	{
		case ECtlCmdButPanel:
		{
			const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, Control(aControlId))->StateChangeControlId();
			switch (ctrlId)
			{
				case ECtlCmdButRun:
					TryExitL(EEikBidOk);
					break;
				case ECtlCmdButCancel:
					TryExitL(EEikBidCancel);
					break;
				case ECtlCmdButInfo:
//					ShowGameDetailsL(GameIndex());
					break;
			}
			break;
		}
	}
}

void CPlayListDialog::PreLayoutDynInitL()
{
#if 0
	TBuf8<128>	gameName8;
	TBuf<128>	gameName;

	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlListboxPlayList);
	CDesCArray* listboxArray=((CDesCArray*)listbox->Model()->ItemTextArray());

	// append contents of the model's checksum array to listbox for display
	TInt count = iGameEngine.GameCount();
	for (TInt i=0 ; i<count ; i++)
	{
		TDesC&	gameName = iGameEngine.GameName(i);
		listboxArray->AppendL(gameName);
	}
	listboxArray->Sort();
#if 0
	if (iGameToRun.Length())
	{
		TInt	index;
		if (listboxArray->FindIsq(iGameToRun, index) == KErrNone)
			listbox->SetCurrentItemIndex(index);
	}
#endif
	CEikScrollBarFrame* scrollbarFrame = listbox->CreateScrollBarFrameL();
	// set horizontal and vertical scroll bars to appear when required
	scrollbarFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
		CEikScrollBarFrame::EAuto);
#endif
}

void CPlayListDialog::GetGameName()
{
	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlListboxPlayList);
	CDesCArray* listboxArray=((CDesCArray*)listbox->Model()->ItemTextArray());
	iGameToRun.Copy(listboxArray->operator[](listbox->CurrentItemIndex()));
}

TBool CPlayListDialog::OkToExitL(TInt /*aKeycode*/)
{
//	GetGameName();
    return(ETrue);
}








CApaDocument* CGameApplication::CreateDocumentL()
{
	CGameDocument*	doc = new (ELeave) CGameDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}



CGameDocument::CGameDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}

CGameDocument::~CGameDocument()
{
	delete iGameEngine;
}

void CGameDocument::ConstructL()
{
	iGameEngine = CGameEngine::NewL(CEikonEnv::Static()->FsSession());
}


CEikAppUi* CGameDocument::CreateAppUiL()
{
	return new(ELeave) CGameAppUi;
}


CGameAppUi::~CGameAppUi()
{
	delete iAppView;
	delete iGameRunner;
}


void CGameAppUi::ConstructL()
{
	BaseConstructL();
	iAppView=new(ELeave) CGameAppView;
	iAppView->ConstructL(ClientRect());

	GameEngine().SetWindowGroupId(CEikonEnv::Static()->RootWin().Identifier());

	SetDefaultGameOptions(IsColorDevice());

	DisplayGamePlayDialogL();
}

void CGameAppUi::SetDefaultGameOptions(TBool aIsColorDevice)
{
	TGameOptions gameOptions = GameEngine().GameOptions();

	gameOptions.iSoundEnabled = EFalse;
	
	if (aIsColorDevice)
	{
		gameOptions.iGameSize = TGameOptions::EGameSizeNormal;
	}
	else
	{
		gameOptions.iOrientation = TGameOptions::EOrientationLeft;
		gameOptions.iContrast = TGameOptions::EContrastSetToLow;
	}
	GameEngine().SetGameOptions(gameOptions);
}



TBool CGameAppUi::IsColorDevice()
{
	TDisplayMode	displayMode = CEikonEnv::Static()->ScreenDevice()->DisplayMode();

	return (displayMode >= EColor16);
}

void CGameAppUi::HandleApplicationSpecificEventL(TInt aType, const TWsEvent& /*aEvent*/)
{
	switch (aType)
	{
#if 0
		case EGameShuttingDown:
//			GameEngine().ShutdownGame();
			StopGame();
//			DisplayGamePlayDialogL();		// put up dialog again
			break;
#endif
		default:
			break;
	}
}


void CGameAppUi::DisplayHelpDialogL()
{
	CEikDialog* dialog = new CGameHelpDialog();

//	dialog->ExecuteLD(R_DIALOG_DIPLAY_EMAME_HELP);
}


void CGameAppUi::HandleCommandL(TInt aCommand)
{
	switch (aCommand)
	{
		case EEikCmdExit: 
			Exit();
			break;

		case EEikCmdGamePlay: 
			DisplayGamePlayDialogL();
			break;

		case EEikCmdGameOptions:
			DisplayGameOptionsDialogL();
			break;

		case EEikCmdHelpPlayGame:
			DisplayHelpDialogL();
			break;

		case EEikCmdHelpAbout:
			DisplayAboutDialogL();
			break;
	}
}




void CGameAppUi::DisplayGameOptionsDialogL()
{
	TGameOptions	gameOptions = GameEngine().GameOptions();
	CEikDialog* dialog = new CGameOptionsDialog(gameOptions, IsColorDevice());

	if (dialog->ExecuteLD(R_DIALOG_GAME_OPTIONS))
	{
		GameEngine().SetGameOptions(gameOptions);
	}
}


void CGameAppUi::DisplayGamePlayDialogL()
{
	CEikDialog*	dialog = new CPlayListDialog(GameEngine(), iGameToRun);
	if (dialog->ExecuteLD(R_DIALOG_PLAY_LIST))
	{
		RunGameL();
	}
}


void CGameAppUi::RunGameL()
{
	TFileName		gameFileName;
//	if (GameEngine().GetGameFileLocation(iGameToRun, gameFileName))
	{
		delete iGameRunner;
		iGameRunner = 0;

		iGameRunner = CGameRunner::NewL(*this, GameEngine());
		iGameRunner->RunGameL(gameFileName, GameEngine().GameOptions());
		
		iGameRunningDialog = new CGameRunningDialog(iGameToRun, GameEngine());
		iGameRunningDialog->ExecuteLD(R_DIALOG_RUNNING_GAME);
		
		DisplayGamePlayDialogL();
	};
}


void CGameAppUi::StopGame()
{
	iGameRunner->TerminateGame();
	if (iGameRunningDialog)
	{
		delete iGameRunningDialog;
		iGameRunningDialog = NULL;
	}
		/*
		 * Bring ourselves forward
		 */
	CEikonEnv::Static()->RootWin().SetOrdinalPosition(0);
}

void CGameAppUi::NotifyGameFinished()
{
	StopGame();
	delete iGameRunner;
	iGameRunner = 0;
}


void CGameAppUi::DisplayAboutDialogL()
{
	CEikDialog*	dialog = new CAboutDialog;
	dialog->ExecuteLD(R_DIALOG_ABOUT);
}



void CGameAppView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SET_RECT(aRect);
	ActivateL();

		/*
		 * Load background image; either grayscale or color depending on the device
		 * we're running on.
		 */
#if 0
	TInt	image = CGameAppUi::Static()->IsColorDevice() ? EMbmEmameappEmame256 : EMbmEmameappEmameg16;
	TRAPD(err, iBgImage = CEikonEnv::Static()->CreateBitmapL(_L("c:\\emame\\app\\emameapp.mbm"), image));
	if (err)
	{
		TRAP(err, iBgImage = CEikonEnv::Static()->CreateBitmapL(_L("d:\\emame\\app\\emameapp.mbm"), image));
	}
	if (iBgImage)
	{
		iImagePosition.iX = (aRect.Size().iWidth - iBgImage->SizeInPixels().iWidth) / 2;
		iImagePosition.iY = (aRect.Size().iHeight - iBgImage->SizeInPixels().iHeight) / 2;
	}
#endif
}

CGameAppView::~CGameAppView()
{
	delete iBgImage;
}


void CGameAppView::Draw(const TRect& /*aRect*/) const
{
#if 0
	CWindowGc& gc = SystemGc();

	if (iBgImage)
		gc.DrawBitmap(iImagePosition, iBgImage);
#endif
}


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CGameApplication;
}



TUid CGameApplication::AppDllUid() const
{
	return KUidGameApp;
}
