/*******************************************************************
 *
 *	File:		EGameApp.h
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAME_APP_H
#define __GAME_APP_H

#include <coecntrl.h>
#include <coeccntx.h>
#include <coemain.h>

#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>

#ifndef __ER6__
#include <eikcmds.hrh>
#endif


#include "GameApp.hrh"
#include "GameEngine.h"


const TUid KUidGameApp = { 0x10009BB4 };


class CWsBitmap;

class CGameDocument : public CEikDocument
{
public:
	~CGameDocument();
	CGameDocument(CEikApplication& aApp);
	void ConstructL();

	CGameEngine&	GameEngine(){return *iGameEngine;}

protected:
	CGameEngine*	iGameEngine;

private: // from CEikDocument
	CEikAppUi* CreateAppUiL();
};


class CGameAppView : public CCoeControl, public MCoeControlBrushContext
{
public:
	~CGameAppView();
	void ConstructL(const TRect& aRect);
private:
	void Draw(const TRect& /*aRect*/) const;

	CWsBitmap*	iBgImage;
	TPoint		iImagePosition;
};



class CGameAppUi : public CEikAppUi, MGameWatcher
{
public:
	CGameAppUi(){}
	void ConstructL();
	~CGameAppUi();

	TBool CGameAppUi::IsColorDevice();

		// cannot be called during construction of the document!
	static CGameAppUi*	Static() {return (CGameAppUi*) CEikonEnv::Static()->AppUi();}

protected: // from CEikAppUi
	virtual void HandleApplicationSpecificEventL(TInt aType,const TWsEvent& aEvent);

	void SetDefaultGameOptions(TBool aIsColorDevice);
	void HandleCommandL(TInt aCommand);
	void DisplayGamePlayDialogL();
	void DisplayGameOptionsDialogL();
	void DisplayHelpDialogL();
	void DisplayAboutDialogL();

	CGameEngine& GameEngine(){return ((CGameDocument*) Document())->GameEngine();}
	CGameRunner*	iGameRunner;
	CEikDialog*		iGameRunningDialog;

	void StartGameL(TInt aGame);
	void StopGame();

protected:
	virtual void NotifyGameFinished();

protected:
	TGameName		iGameToRun;

	void RunGameL();

private:
	CGameAppView* iAppView;
};






class CGameApplication : public CEikApplication
	{
private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
	};

#endif
