/*******************************************************************
 *
 *	File:		GameEngine.h
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __GAME_ENGINE_H
#define __GAME_ENGINE_H

#include <e32std.h>
#include <e32base.h>

#include "GlobalData.h"

typedef TBuf<16>	TGameName;
typedef void (*TMainFunc)(int argc, char*argv[]);


class CGameEngine;

class MGameWatcher
{
public:
	virtual void NotifyGameFinished() = 0;
};


class CGameRunner : public CActive
{
public:
	static CGameRunner* NewL(MGameWatcher& aGameWatcher, CGameEngine& aGameEngine);
	~CGameRunner();

	void RunGameL(const TDesC& aGameName, const TGameOptions& aGameOptions);
	void TerminateGame();

protected:
	CGameRunner(MGameWatcher& aGameWatcher, CGameEngine& aGameEngine);
	void ConstructL();

	virtual void RunL();
	virtual void DoCancel();

	void ShutdownGame();
	void StartGameL(const TDesC& aGameName);

#ifdef __WINS__
	static TInt ThreadGame(TAny* aData);
	void DoRunGameL();
#endif
	
	MGameWatcher&		iGameWatcher;
	TFileName			iGameName;
	TInt				iGameCounter;
	CGameEngine&		iGameEngine;

	char*				iArgv[64];
	int					iArgc;
	TCommand			iCommandLine;
	TBuf8<KMaxFileName>	iFileName;

#ifdef __WINS__
	RThread				iChild;
#else
	RProcess			iChild;
#endif


#ifdef __WINS__
	RLibrary	iGameLib;
	TMainFunc	iMainFunc;
#endif
};




class CGameEngine : public CBase
{
public:
	static CGameEngine* NewL(RFs& aFs);
	~CGameEngine();

	TGameOptions&		GameOptions() {return iGameOptions;}
	void				SetGameOptions(const TGameOptions& aGameOptions){iGameOptions = aGameOptions;}
	void				SetWindowGroupId(TInt aWindowGroupId);
	void				ShutdownGame();
	TInt				GameNo(const TDesC& aGameName);
	TDesC&				GameName(TInt aGameNo);
	TBool				IsGameCloneOf(struct GameDriver* aGame);
	void				ForceGameExit();
	TInt				GameCount();
	TBool				GetGameFileLocation(const TDesC& aGameName, TDes& aGameFileName);
	void				KillRunningStdoutConsole();

protected:
	CGameEngine(RFs& aFs);
	void ConstructL();


	void AddGames(CDir* aEntryList);

	TGameOptions	iGameOptions;

	RChunk			iGlobalDataChunk;

	RFs&			iFs;

	CArrayFixFlat<TGameName>* iGameNameArray;
};


#endif			/* __GAME_ENGINE_H */
