/*******************************************************************
 *
 *	File:		Video.h
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __VIDEO_H
#define __VIDEO_H

#include "GlobalData.h"

struct osd_bitmap
{
	TInt width,height;	/* width and height of the bitmap */
	TInt depth;			/* bits per pixel */
	void *_private;		/* don't touch! - reserved for osdepend use */
	TUint8 **line;		/* pointers to the start of each line */
};


#include <e32base.h>
#include <w32std.h>

class MKeyboardObserver
{
public:
	virtual void KeyDownEvent(TInt aScancode) = 0;
	virtual void KeyUpEvent(TInt aScancode) = 0;
};


#define MAX_ONSCREENCONTROLS	4
#define MAX_KEYPRESS			5

const TInt KSizeGameColorPalette = 256;

enum TOrientation
{
	EOrientationNormal,
	EOrientationRotLeft,
	EOrientationRotRight
};


class TOnScreenControlList
{
public:
	TInt	iKey;
	TRect	iRect;
};


class CGameFbsBitmap : public CFbsBitmap
{
public:
	CBitwiseBitmap* GameAddress() const {return iAddressPointer;}
};


class CGameBitmap : public CBase
{
public:
	~CGameBitmap();
	static CGameBitmap* NewL(TInt aWidth, TInt aHeight, TDisplayMode aDisplayMode);

	struct osd_bitmap*	OsdBitmap() {return &iOsdBitmap;}
	struct osd_bitmap*	OsdVisibleBitmap() {return &iOsdVisibleBitmap;}
	CFbsBitmap*			FbsBitmap(){return iBitmap;}
	void				SetVisibleAreaL(const TRect& aVisibleArea);

	static TDisplayMode GetDisplayMode(TInt aDepth, TBool aColor = ETrue);
	static TInt			GetBpp(TDisplayMode aDisplayMode);
	static TInt			GetColorDepth(TDisplayMode aDisplayMode);

	
protected:
	CGameBitmap();
	void ConstructL(TInt aWidth, TInt aHeight, TDisplayMode aDisplayMode);

	struct osd_bitmap		iOsdBitmap;
	struct osd_bitmap		iOsdVisibleBitmap;

	CGraphicsContext*		iBitmapContext;
	CGameFbsBitmap*			iBitmap;
	CFbsBitmapDevice*		iBitmapDevice;
};


typedef RPointerArray<CGameBitmap>	CGameBitmapArray;
const TInt							KGameBitmapArrayGranularity = 5;

class CGameScreen : public CBase
{
public:
	CGameScreen(){};
	~CGameScreen();
	CGameScreen(const TScreenInfoV01& aScreenInfo, TDisplayMode aDisplayMode);
	virtual void ConstructL(TInt aWidth, TInt aHeight, TInt aDepth, TOrientation aOrientation, TInt aFps);

	void DrawBitmap(CGameBitmap* aGameBitmap){(this->*iBitmapDrawer)(aGameBitmap);}
	void AllocateColors(TUint totalcolors, const TUint8 *palette, TUint16 *pens, TInt modifiable, const TUint8 *debug_palette,TUint16 *debug_pens);

	TDisplayMode	DisplayMode(){return iDisplayMode;}
	TOrientation	Orientation(){return iOrientation;}
	TBool			HasFrameBuffer(){return iHasFrameBuffer;}
	void			SetPosition(const TPoint& aPosition) {iPosition = aPosition;}
	TPoint			Position() {return iPosition;}
	TSize			ScreenSize() {return iScreenSize;}
	TSize			GameBitmapSize(){return TSize(iWidth, iHeight);}
	TBool			DoubleFrame(TInt aShowFps, struct osd_bitmap *bitmap);
	TInt			SkipThisFrame();
	void			FrameTick();
	TInt			Fps(){return iGameFps;}
	TInt			TotalGameFrames() {return iTotalGameFrames;}

	void			SetWantScaling(TBool aWantScaling);
	void			SetWantDoubleBitmap(TBool aWantDoubleBitmap);

	TUint8*			FrameBufferAddress(){return iFrameBuffer;}
	TInt			FrameBufferBytesPerScanLine(){return iBytesPerScanLine;}
	TInt			FindEpocRgbColor(int r, int g, int b);

protected:
	typedef void (CGameScreen::*TScaledScanLineDrawer)(TUint* aSrcLine, TUint* aDstLine, TInt aWidth, TBool aMerge);
	TScaledScanLineDrawer	iScaledScanLineDrawer;

	typedef void (CGameScreen::*TBitmapDrawer)(CGameBitmap* aGameBitmap);
	TBitmapDrawer	iBitmapDrawer;

	void DrawScaledScanLine8bppTo2bpp(TUint* aSrcLine, TUint* aDstLine, TInt aWidth, TBool aMerge);
	void DrawScaledScanLine8bppTo4bpp(TUint* aSrcLine, TUint* aDstLine, TInt aWidth, TBool aMerge);
	void DrawScaledScanLine8bppTo8bpp(TUint* aSrcLine, TUint* aDstLine, TInt aWidth, TBool aMerge);
	void DrawScaledScanLine8bppTo12bpp(TUint* aSrcLine, TUint* aDstLine, TInt aWidth, TBool aMerge);

	void DrawDoubleScaledScanLine8bppTo8bpp(TUint* aSrcLine, TUint* aDstLine, TInt aWidth, TBool aMerge);

	void DirectDrawBitmap8bppTo2bpp(CGameBitmap* aGameBitmap);
	void DirectDrawBitmap8bppTo4bpp(CGameBitmap* aGameBitmap);
	void DirectDrawBitmap8bppTo8bpp(CGameBitmap* aGameBitmap);
	void DirectDrawBitmap8bppTo12bpp(CGameBitmap* aGameBitmap);
	void ScaledDrawBitmap(CGameBitmap* aGameBitmap);	// uses ScaledDrawScanLineXbppToYbpp

	void DirectDrawDoubleBitmap8bppTo8bpp(CGameBitmap* aGameBitmap);
	void DoubleScaledDrawBitmap(CGameBitmap* aGameBitmap);

	void	SetBitmapPosition(const TPoint& aPosition);
	void	SetupScreenAccess();
	void	SetupScaling();
	void	SetupPosition();

	TInt	AllocateGamePen(const TRgb& aColor);
	TUint	GetColorValue(const TRgb& aColor, const TDisplayMode& aDisplayMode);
	void	AllocateGameColors(TUint totalcolors, const TUint8 *palette, TUint16 *pens, TInt modifiable, const TUint8 *debug_palette,TUint16 *debug_pens);
	void	AllocateEpocColors(TUint totalcolors, const TUint8 *palette, TUint16 *pens, TInt modifiable, const TUint8 *debug_palette,TUint16 *debug_pens);

	TInt				iGamePaletteMaxIndex;
	TUint				iGamePalette[KSizeGameColorPalette];

	TUint8*				iScanLine;

	TBool				iHasFrameBuffer;
	TInt				iBytesPerBitmapLine;
	TInt				iBytesPerScanLine;
	TDisplayMode		iDisplayMode;
	TSize				iScreenSize;
	TUint8*				iBitmapStartAddress;

	TInt				iWidth;
	TInt				iHeight;
	TInt				iGameDepth;
	TInt				iGameWidth;
	TInt				iGameHeight;
	TInt				iPixelSkipX;
	TInt				iPixelSkipY;
	TPoint				iPosition;

	TBool				iWantPanning;
	TBool				iWantScaling;
	TBool				iWantDoubleBitmap;
	TBool				iDoubleBitmap;
	TBool				iWantMaintainRatioWhenScaling;
	TBool				iFirstTime;

	TOrientation		iOrientation;

	TPoint				iBitmapPosition;
	TUint8*				iFrameBuffer;		// if NULL we can't do direct screen access

/**************************************
 *
 * Frame skip + throttle
 *
 **************************************/
	TInt		iVideoFps;
	TInt		iPeriod;
	TInt		iDoubleFrame;
	TInt		iDoubleFrameCount;
	TInt		iRealFrameCount;
	TInt		iGameFrameCount;
	TInt		iPrevTicks;

	TInt		iTicksGameStart;
	TInt		iTotalRealFrames;

	TInt		iDiffFrames;
	TInt		iRealFps;

	TInt		iGameFps;
	TInt		iGameFrames;
	TInt		iTotalGameFrames;
};


#ifdef __WINS__

	/*
	 * Sothat we can test the target drawing routines under target
	 */
class CWinsGameScreen : public CGameScreen
{
public:
	CWinsGameScreen(){}
	CWinsGameScreen(const TSize& aScreenSize, TDisplayMode aDisplayMode);
	~CWinsGameScreen();
	virtual void ConstructL(TInt aWidth, TInt aHeight, TInt aDepth, TOrientation aOrientation, TInt aFps);
	CGameBitmap*	iEmulatedScreenBitmap;
};
#endif


class CGameWindow : public CBase
{
public:
	~CGameWindow();
	static CGameWindow* NewL();

	void CreateDisplayL(TInt aWidth, TInt aHeight, TInt aDepth, TOrientation aOrientation, TInt aFps);
	void CloseDisplay();
	void SetVisbibleAreaL(const TRect aRect);

	CGameBitmap*	AllocateBitmapL(TInt aWidth, TInt aHeight, TInt aDepth);
	void DrawBitmap(CGameBitmap* aGameBitmap);

	void MarkInvalid(TInt aXmin, TInt aYmin, TInt aXmax, TInt aYmax);
	TInt SkipThisFrame();

	void AllocateColor(TUint totalcolors, const TUint8 *palette, TUint16 *pens, TInt modifiable, const TUint8 *debug_palette,TUint16 *debug_pens);

	void SetKeyboardObserver(MKeyboardObserver* aKeyboardObserver);
	TBool IsKeyPressed(TInt aKey);
	void Pause(TBool aPause);
	void PollWServEvents();

	CGameScreen*	GameScreen(){return iGameScreen;}
	void EmulateTargetDrawBitmap(CGameBitmap* aGameBitmap);

protected:
	CGameWindow();
	void ConstructL();
	void Destruct();

	void HandleWsEvent(const TWsEvent& aWsEvent);
	void HandleRedrawEvent(const TWsRedrawEvent& iRedrawEvent);
	void StartWServEvents();
	void CancelWServEvents();
	void WsBlitBitmap(CGameBitmap* aGameBitmap);
	void DrawOnScreenControlsNow();

	CGameBitmapArray*		iGameBitmapArray;

	TBool					iShowFps;

	CGameScreen*		iGameScreen;
	void				CreateGameScreenL(TInt aWidth, TInt aHeight, TInt aDepth, TOrientation aOrientation, TInt aFps);

	TInt				iLastOnScreenKey;
	TBool				iIsPaused;

	TOnScreenControlList	iOnScreenControlList[MAX_ONSCREENCONTROLS];
	void SetupOnScreenControls();
	void DrawOnScreenControls();
	TInt GetOnScreenKey(const TPoint aPosition);
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);

		
		/*
		 * Maintain state about key presses. A key is pressed from the moment we get
		 * its keydown event until we get its keyup event. While holding down a key
		 * another key may be pressed. Hence we use this array below to keep track
		 * of these nested key presses.
		 */
	struct TKeyPress
	{
		TInt	iScanCode;
		TInt	iKeyCode;
	};

	TKeyPress	iKeyPress[MAX_KEYPRESS];
	TInt		iKeyPressLevel;
	void KeyDownEvent(TInt aScanCode);
	void KeyPressEvent(TInt aKeyCode);
	void KeyUpEvent(TInt aScanCode);
	TInt CurrentKey();

	MKeyboardObserver*	iKeyboardObserver;

		/*
		 * Window Server stuff
		 */
	RWsSession			iWsSession;
	RWindowGroup		iWsWindowGroup;
	RWindow				iWsWindow;
	CWsScreenDevice*	iWsScreen;
	CWindowGc*			iWindowGc;

	TRequestStatus		iWsEventStatus;
	TRequestStatus		iRedrawEventStatus;
	TWsEvent			iWsEvent;
	TWsRedrawEvent		iRedrawEvent;
};



#endif			/* __VIDEO_H */
