/*******************************************************************
 *
 *	File:		EDoom.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


extern "C"
{
#include "doomdef.h"
#include "i_system.h"
#include "m_argv.h"
#include "st_stuff.h"
#include "i_video.h"
#include "z_zone.h"
#include "i_joy.h"
#include "m_misc.h"


/*********************************************************
 *
 * Forward declarations of simple helper routines
 *
 *********************************************************/

int stricmp( const char *string1, const char *string2 );
int strnicmp( const char *string1, const char *string2, int n);
void* alloca(size_t size);
int strupr(char *n);
int strlwr(char *n);

void* my_malloc(size_t);
void* my_calloc(size_t, size_t);

/*********************************************************
 *
 * Global variables we are supposed to declare
 *
 *********************************************************/

// this is the CURRENT rendermode!! very important : used by w_wad, and many other code
FILE*			debugfile;
byte			mb_used = 4;
JoyType_t		Joystick;



/*********************************************************
 *
 * Routines that we will never implement
 *
 *********************************************************/

void I_ShutdownCD (void){}
void I_InitCD (void){}
void I_UpdateCD (void){}
void I_PlayCD (int /*nTrack*/, boolean /*bLooping*/){}
void I_StopCD (void){}
void I_ResumeCD (void){}
int I_SetVolumeCD (int /*volume*/){return 0;}

void I_InitJoystick (void) {}
void I_StartupMouse(void) {}
void I_StartupMouse2(void) {}
void I_StartupTimer(void) {}
void I_StartupKeyboard (void) {}
int I_GetKey (void) { return 0; }
void I_GetFreeMem(void) {}
void I_GetDiskFreeSpace(INT64 *freespace){*freespace = MAXINT;}
void I_Tactile (int /*on*/, int /*off*/, int /*total*/){}
ticcmd_t emptycmd;
ticcmd_t* I_BaseTiccmd(void){return &emptycmd;}

void I_InitMusic(void){}
int I_RegisterSong(void* /*data*/,int /*len*/){return 0;}
void I_PlaySong(int /*handle*/, int /*bLooping*/){}
//void I_SetSfxVolume(int /*volume*/){}
void I_SetMusicVolume(int /*volume*/){}
void I_ResumeSong (int /*handle*/){}
void I_PauseSong (int /*handle*/){}
//void* I_GetSfx (sfxinfo_t*  /*sfx*/){return NULL;}
//void  I_FreeSfx (sfxinfo_t* /*sfx*/){}
void I_StopSong(int /*handle*/){}
void I_UnRegisterSong(int /*handle*/){}




/****************************************************
 *
 * SYSTEM ROUTINES
 *
 ****************************************************/


void I_StartTic()
{
	// check key events etc
}

ULONG  I_GetTime (void);

int  I_mkdir(const char *dirname, int unixright);

char *I_GetUserName(void)
{
	return "peter";
}



byte* I_ZoneBase (int*	size)
{
	*size = mb_used*1024*1024;
	return (byte *) malloc (*size);
}



/****************************************************
 *
 * AUDIO ROUTINES
 *
 ****************************************************/
}		// extern "C"






/*********************************************************
 *
 * Implementation of simple helper routines
 *
 *********************************************************/

#include <e32std.h>
#include <e32svr.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/time.h>

#include "game.h"


int stricmp( const char *string1, const char *string2)
{
	return strcasecmp(string1, string2);
}

int strnicmp( const char *string1, const char *string2, int n)
{
	return strncasecmp(string1, string2, n);
}

void* alloca(size_t size)
{
	RDebug::Print(_L("Alloca: size=%d"), size);
	// memory leak, but alas
	return malloc(size);
}

int strupr(char *n) 
{
      int i;
      for (i=0;n[i];i++) {
               n[i] = toupper(n[i]);
      }
      return 1;
}

int strlwr(char *n) 
{
      int i;
      for (i=0;n[i];i++) {
               n[i] = tolower(n[i]);
      }
      return 1;
}



ULONG  I_GetTime (void)
{
    struct timeval	tp;
    struct timezone	tzp;
    int			newtics;
    static int		basetime=0;

    gettimeofday(&tp, &tzp);
    if (!basetime)
	basetime = tp.tv_sec;
    newtics = (tp.tv_sec-basetime)*TICRATE + tp.tv_usec*TICRATE/1000000;

    return newtics;
}


int I_mkdir(const char *dirname, int unixright)
{
	return mkdir(dirname, unixright);
}


#undef malloc
#undef calloc


void* my_malloc(size_t s)
{
	void*	result;
	result = User::Alloc(s);
	if (!result)
	{
		User::InfoPrint(_L("no memory"));
		User::After(3 * 1000 * 1000);
	}
//	RDebug::Print(_L("my_malloc, size=%d"), s);
	return result;
}

void* my_calloc(size_t num, size_t size)
{
	void*	result;
	result = User::Alloc(num * size);
	if (result)
		Mem::FillZ(result, num*size);
	if (!result)
	{
		User::InfoPrint(_L("no memory"));
		User::After(3 * 1000 * 1000);
	}
//	RDebug::Print(_L("my_calloc, size=%d"), num*size);
	return result;
}


void DebugPrint(const char* str, int sec)
{
	TPtrC8	ptr((TText8*)str);
	User::InfoPrint(ptr);
	User::After(1000 * 1000 * sec);
}
void I_Error (char *error, ...)
{
    va_list     argptr;

    // Message first.
    va_start (argptr,error);
    fprintf (stderr, "Error: ");
    vfprintf (stderr,error,argptr);
    fprintf (stderr, "\n");
    va_end (argptr);

	TPtrC8	ptr((TText8*) error);
	RDebug::Print(_L("I_Error: %S"), &ptr);;

    fflush( stderr );

//    I_ShutdownMusic();
//    I_ShutdownSound();
//    I_ShutdownGraphics();
    
    exit(-1);
}

void I_Quit (void)
{
	// use this for 1.28 19990220 by Kin
	M_SaveConfig (NULL);

#ifdef __MARM__
	RProcess	me;
	me.Terminate(0);
#else
	exit(0);
#endif
}
