/*******************************************************************
 *
 *	File:		EDoomMain.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <e32std.h>
#include <f32file.h>
#include <e32svr.h>
#include <estlib.h>
#include <eikdll.h>
#include <w32std.h>

#include "game.h"

extern "C"
{
#include "doomdef.h"
#include "m_argv.h"
#include "d_main.h"

int doom_main(int argc, char** argv) 
{ 
	myargc = argc; 
	myargv = argv; 

	D_DoomMain (); 
	DPRINT(("doom_main: entering doom loop"));
	D_DoomLoop ();
	return 0;
} 

}


void EDoomDebugPrint(const char* format, ...)
{
	char	text[128];
	va_list     argptr;

	va_start(argptr, format);
	vsprintf(text, format, argptr);
	va_end(argptr);

	TPtrC8	ptr((TText8*) text);
	RDebug::Print(ptr);
}


void EpocGameL(int argc, char* argv[])
{
#ifdef __WINS__
	int crt_argc=0;
	char** crt_argv=0;
	char** crt_envp=0;

	__crt0(crt_argc, crt_argv, crt_envp);
#endif
	printf(" ");
	fflush(stdout);


	if (chdir("d:\\edoom") != 0)
		chdir("c:\\edoom");

#if 0
#ifdef __WINS__
	setenv("DOOMWADDIR", "c:\\edoom", 1);
#else
	setenv("DOOMWADDIR", "d:\\edoom", 1);
#endif
#endif

	doom_main(argc, argv);

#ifdef __WINS__
	RThread	me;
#else
	RProcess	me;
	me.Terminate(0);
#endif

#ifdef __WINS__
	fclose(stdin);
	fclose(stdout);
	fclose(stderr);
#endif
}







#ifdef __WINS__

EXPORT_C void EntryRunGame(int argc, char* argv[])
{
	EpocGameL(argc, argv);
}


int __cdecl E32Dll(TDllReason aReason)
{
	return(KErrNone);
}

int main(int argc, char* argv[])
{
	DEBUGGER();
	return 0;
}



#else

#define APP_ON_C "C:\\DOOM\\APP\\DOOMAPP.APP"
#define APP_ON_D "D:\\DOOM\\APP\\DOOMAPP.APP"

void StartGameAppL()
{
	CApaCommandLine* apaCommandLine = CApaCommandLine::NewLC();
	if (access(APP_ON_D, 0) != -1)
		apaCommandLine->SetLibraryNameL(_L(APP_ON_D));
	else
		apaCommandLine->SetLibraryNameL(_L(APP_ON_C));

	CleanupStack::Pop();
	EikDll::StartAppL(*apaCommandLine);
}


void TargetEpocGameL(int argc, char* argv[])
{
	if (argc == 1)
	{
#if 0
		if (!IsGameAppRunning())
			StartGameAppL();
		else
			User::InfoPrint(_L("EDoom is already running"));
		return;
#endif
	}

	EpocGameL(argc, argv);
}


int TargetGame(int argc, char* argv[])
{
	CTrapCleanup* cleanup=CTrapCleanup::New();

	TRAPD(error, TargetEpocGameL(argc, argv));

	__ASSERT_ALWAYS(!error, User::Panic(_L("EDOOM"), error));

	delete cleanup;

	return 0;
}


extern "C" {
int main(int argc, char* argv[])
{
	TargetGame(argc, argv);

	return 0;
}
}		// extern "C"

#endif

