/*******************************************************************
 *
 *	File:		i_audio.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#include "audio.h"

static CGameAudio* sGameAudio= NULL;

extern "C"
{
#include "doomdef.h"
#include "i_system.h"
#include "m_argv.h"
#include "st_stuff.h"
#include "i_video.h"
#include "z_zone.h"
#include "m_misc.h"


#include "doomstat.h"
#include "s_sound.h"
consvar_t cdUpdate  = {"cd_update","1",CV_SAVE};
#include "i_sound.h"
extern CV_PossibleValue_t soundvolume_cons_t[];
consvar_t   cd_volume = {"cd_volume","31",CV_SAVE,soundvolume_cons_t};

}		// extern "C"




/**************************************
 *
 * AUDIO
 *
 **************************************/

// Number of sound channels used as set in the config file
extern  consvar_t cv_numChannels;

// Flags for the -nosound and -nomusic options
extern boolean nosound;
extern boolean nomusic;

// A quick hack to establish a protocol between
// synchronous mix buffer updates and asynchronous
// audio writes. Probably redundant with gametic.
volatile static int flag = 0;

// The number of internal mixing channels,
//  the samples calculated for each mixing step,
//  the size of the 16bit, 2 hardware channel (stereo)
//  mixing buffer, and the samplerate of the raw data.


// Needed for calling the actual sound output.
//#define SAMPLECOUNT             1024
#define SAMPLECOUNT             128
#define NUM_CHANNELS            16
// It is 2 for 16bit, and 2 for two channels.
#define BUFMUL                  4
#define MIXBUFFERSIZE           (SAMPLECOUNT*BUFMUL)

#define SAMPLERATE              11025   // Hz
#define SAMPLESIZE              2       // 16bit

// The actual output device and a flag for using 8bit samples.
//int     audio_fd;
int     audio_8bit_flag = 1;		// EPOC

// The global mixing buffer.
// Basically, samples from all active internal channels
//  are modifed and added, and stored in the buffer
//  that is submitted to the audio device.
signed short    mixbuffer[MIXBUFFERSIZE];


// The channel step amount...
unsigned int    channelstep[NUM_CHANNELS];
// ... and a 0.16 bit remainder of last step.
unsigned int    channelstepremainder[NUM_CHANNELS];


// The channel data pointers, start and end.
unsigned char*  channels[NUM_CHANNELS];
unsigned char*  channelsend[NUM_CHANNELS];


// Time/gametic that the channel started playing,
//  used to determine oldest, which automatically
//  has lowest priority.
// In case number of active sounds exceeds
//  available channels.
int             channelstart[NUM_CHANNELS];

// The sound in channel handles,
//  determined on registration,
//  might be used to unregister/stop/modify,
int             channelhandles[NUM_CHANNELS];

// SFX id of the playing sound effect.
// Used to catch duplicates (like chainsaw).
int             channelids[NUM_CHANNELS];                       

// Pitch to stepping lookup, unused.
int             steptable[256];

// Volume lookups.
int             vol_lookup[128*256];

// Hardware left and right channel volume lookup.
int*            channelleftvol_lookup[NUM_CHANNELS];
int*            channelrightvol_lookup[NUM_CHANNELS];

// master hardware sound volume
int		hw_sndvolume=31;



// dummy for now 19990220 by Kin
void I_FreeSfx(sfxinfo_t* /*sfx*/) 
{
}

//
// This function loads the sound data from the WAD lump,
//  for single sound.
//
void* I_GetSfx (sfxinfo_t*  sfx)
{
    byte*               dssfx;
    int                 size,i;
    unsigned char*      paddedsfx;
    int                 paddedsize;

    if (sfx->lumpnum<0)
    {
        sfx->lumpnum = S_GetSfxLumpNum (sfx);
    }

    size = W_LumpLength (sfx->lumpnum);

    dssfx = (byte*) W_CacheLumpNum (sfx->lumpnum, PU_STATIC);
    paddedsize = ((size-8 + (SAMPLECOUNT-1)) / SAMPLECOUNT) * SAMPLECOUNT;
    paddedsfx = (unsigned char*)Z_Malloc( paddedsize+8, PU_STATIC, 0 );
    memcpy(  paddedsfx, dssfx, size );
    for (i=size ; i<paddedsize+8 ; i++)
        paddedsfx[i] = 128;

    // Remove the cached lump.
    Z_Free( dssfx );
    *((int*)paddedsfx) = paddedsize;
    return (void*)(paddedsfx);

}




//
// This function adds a sound to the
//  list of currently active sounds,
//  which is maintained as a given number
//  (eight, usually) of internal channels.
// Returns a handle.
//
int
addsfx
( int           sfxid,
  int           volume,
  int           step,
  int           seperation )
{
    static unsigned short       handlenums = 0;
 
    int         i;
    int         rc = -1;
    
    int         oldest = gametic;
    int         oldestnum = 0;
    int         slot;

    int         rightvol;
    int         leftvol;

    // Chainsaw troubles.
    // Play these sound effects only one at a time.
    if ( sfxid == sfx_sawup
         || sfxid == sfx_sawidl
         || sfxid == sfx_sawful
         || sfxid == sfx_sawhit
         || sfxid == sfx_stnmov
         || sfxid == sfx_pistol  )
    {
        // Loop all channels, check.
        for (i=0 ; i<cv_numChannels.value ; i++)
        {
            // Active, and using the same SFX?
            if ( (channels[i])
                 && (channelids[i] == sfxid) )
            {
                // Reset.
                channels[i] = 0;
                // We are sure that iff,
                //  there will only be one.
                break;
            }
        }
    }

    // Loop all channels to find oldest SFX.
    for (i=0; (i<cv_numChannels.value) && (channels[i]); i++)
    {
        if (channelstart[i] < oldest)
        {
            oldestnum = i;
            oldest = channelstart[i];
        }
    }

    // Tales from the cryptic.
    // If we found a channel, fine.
    // If not, we simply overwrite the first one, 0.
    // Probably only happens at startup.
    if (i == cv_numChannels.value)
        slot = oldestnum;
    else
        slot = i;

    // Okay, in the less recent channel,
    //  we will handle the new SFX.
    // Set pointer to raw data.
    channels[slot] = (unsigned char *) S_sfx[sfxid].data+8;
    // Set pointer to end of raw data.
    channelsend[slot] = channels[slot] + *((int*)S_sfx[sfxid].data);

    // Reset current handle number, limited to 0..100.
    //if (!handlenums)
    //  handlenums = 100;

    // Assign current handle number.
    // Preserved so sounds could be stopped (unused).
    channelhandles[slot] = rc = handlenums++;

    // Set stepping???
    // Kinda getting the impression this is never used.
    channelstep[slot] = step;
    // ???
    channelstepremainder[slot] = 0;
    // Should be gametic, I presume.
    channelstart[slot] = gametic;

    // Separation, that is, orientation/stereo.
    //  range is: 1 - 256
    seperation += 1;

    // Per left/right channel.
    //  x^2 seperation,
    //  adjust volume properly.
    leftvol =
        volume - ((volume*seperation*seperation) >> 16); ///(256*256);
    seperation = seperation - 257;
    rightvol =
        volume - ((volume*seperation*seperation) >> 16);        

    // Sanity check, clamp volume.
    if (rightvol < 0 || rightvol > 127)
        I_Error("rightvol out of bounds");
    
    if (leftvol < 0 || leftvol > 127)
        I_Error("leftvol out of bounds");
    
    // Get the proper lookup table piece
    //  for this volume level???
    channelleftvol_lookup[slot] = &vol_lookup[(leftvol*hw_sndvolume/31)*256];
    channelrightvol_lookup[slot] = &vol_lookup[(rightvol*hw_sndvolume/31)*256];

    // Preserve sound SFX id,
    //  e.g. for avoiding duplicates of chainsaw.
    channelids[slot] = sfxid;

    // You tell me.
    return rc;
}





//
// SFX API
// Note: this was called by S_Init.
// However, whatever they did in the
// old DPMS based DOS version, this
// were simply dummies in the Linux
// version.
// See soundserver initdata().
//
void I_SetChannels()
{
  // Init internal lookups (raw data, mixing buffer, channels).
  // This function sets up internal lookups used during
  //  the mixing process. 
  int           i;
  int           j;
    
  int*  steptablemid = steptable + 128;
  
  // Okay, reset internal mixing channels to zero.
  /*for (i=0; i<cv_numChannels.value; i++)
  {
    channels[i] = 0;
  }*/

  // This table provides step widths for pitch parameters.
  // I fail to see that this is currently used.
  for (i=-128 ; i<128 ; i++)
    steptablemid[i] = (int)(pow(2.0, (i/64.0))*65536.0);
  
  
  // Generates volume lookup tables
  //  which also turn the unsigned samples
  //  into signed samples.
  for (i=0 ; i<128 ; i++)
  {
    for (j=0 ; j<256 ; j++)
    {
      if (!audio_8bit_flag)
        vol_lookup[i*256+j] = (i*(j-128)*256)/127;
      else
        vol_lookup[i*256+j] = (i*(j-128)*256)/127*4;
    }
  }
}       

 
void I_SetSfxVolume(int volume)
{
  // Identical to DOS.
  // Basically, this should propagate
  //  the menu/config file setting
  //  to the state variable used in
  //  the mixing.
   // dummy for Linux 19990117 by Kin
//  snd_SfxVolume = volume;
  hw_sndvolume = volume;
}



//
// Retrieve the raw data lump index
//  for a given SFX name.
//
int I_GetSfxLumpNum(sfxinfo_t* sfx)
{
    char namebuf[9];
    sprintf(namebuf, "ds%s", sfx->name);
    return W_GetNumForName(namebuf);
}

//
// Starting a sound means adding it
//  to the current list of active sounds
//  in the internal channels.
// As the SFX info struct contains
//  e.g. a pointer to the raw data,
//  it is ignored.
// As our sound handling does not handle
//  priority, it is ignored.
// Pitching (that is, increased speed of playback)
//  is set, but currently not used by mixing.
//
int
I_StartSound
( int           id,
  int           vol,
  int           sep,
  int           pitch,
  int           priority )
{



  // UNUSED
  priority = 0;
  vol = vol >> 4; // xdoom only accept 0-15 19990124 by Kin
  
  if (nosound)
    return 0;

   
    // Returns a handle (not used).
    id = addsfx( id, vol, steptable[pitch], sep );

    // fprintf( stderr, "/handle is %d\n", id );
    
    return id;
}



void I_StopSound (int handle)
{
  // You need the handle returned by StartSound.
  // Would be looping all channels,
  //  tracking down the handle,
  //  an setting the channel to zero.
  
  int i;

  for (i=0; i<cv_numChannels.value; i++)
  {
    if (channelhandles[i] == handle)
    {
      channels[i] = 0;
      break;
    }
  }
}


int I_SoundIsPlaying(int handle)
{
    // Ouch.
    // return gametic < handle;

    int i;

    for (i=0; i<cv_numChannels.value; i++)
    {
      if (channelhandles[i] == handle)
        return 1;
    }
    return 0;
}




//
// This function loops all active (internal) sound
//  channels, retrieves a given number of samples
//  from the raw sound data, modifies it according
//  to the current (internal) channel parameters,
//  mixes the per channel samples into the global
//  mixbuffer, clamping it to the allowed range,
//  and sets up everything for transferring the
//  contents of the mixbuffer to the (two)
//  hardware channels (left and right, that is).
//
void I_UpdateSound( void )
{
  // Debug. Count buffer misses with interrupt.
  static int misses = 0;

  // Flag. Will be set if the mixing buffer really gets updated.
  int updated = 0;
  
  // Mix current sound data.
  // Data, from raw sound, for right and left.
  register unsigned int sample;
  register int          dl;
  register int          dr;
  unsigned short        sdl;
  unsigned short        sdr;
  
  // Pointers in global mixbuffer, left, right, end.
  signed short*         leftout;
  signed short*         rightout;
  signed short*         leftend;
  unsigned char*        bothout;
        
  // Step in mixbuffer, left and right, thus two.
  int                   step;

  // Mixing channel index.
  int                   chan;
  int					epoc_count=0;
    
    // Left and right channel
    //  are in global mixbuffer, alternating.
    leftout = mixbuffer;
    rightout = mixbuffer+1;
    bothout = (unsigned char *)mixbuffer;
    step = 2;

    // Determine end, for left channel only
    //  (right channel is implicit).
    leftend = mixbuffer + SAMPLECOUNT*step;

    // Mix sounds into the mixing buffer.
    // Loop over step*SAMPLECOUNT.
    while (leftout != leftend)
    {
        // Reset left/right value. 
        dl = 0;
        dr = 0;

        // Love thy L2 chache - made this a loop.
        // Now more channels could be set at compile time
        //  as well. Thus loop those  channels.
        for ( chan = 0; chan < cv_numChannels.value; chan++ )
        {
            // Check channel, if active.
            if (channels[ chan ])
            {
                // we are updating the mixer buffer, set flag
                updated++;
                // Get the raw data from the channel. 
                sample = *channels[ chan ];
                // Add left and right part
                //  for this channel (sound)
                //  to the current data.
                // Adjust volume accordingly.
                dl += channelleftvol_lookup[ chan ][sample];
                dr += channelrightvol_lookup[ chan ][sample];
                // Increment index ???
                channelstepremainder[ chan ] += channelstep[ chan ];
                // MSB is next sample???
                channels[ chan ] += channelstepremainder[ chan ] >> 16;
                // Limit to LSB???
                channelstepremainder[ chan ] &= 65536-1;

                // Check whether we are done.
                if (channels[ chan ] >= channelsend[ chan ])
                    channels[ chan ] = 0;
            }
        }
        
        // Clamp to range. Left hardware channel.
        // Has been char instead of short.
        // if (dl > 127) *leftout = 127;
        // else if (dl < -128) *leftout = -128;
        // else *leftout = dl;

        if (!audio_8bit_flag)
        {
            if (dl > 0x7fff)
                *leftout = 0x7fff;
            else if (dl < -0x8000)
                *leftout = -0x8000;
            else
                *leftout = dl;

            // Same for right hardware channel.
            if (dr > 0x7fff)
                *rightout = 0x7fff;
            else if (dr < -0x8000)
                *rightout = -0x8000;
            else
                *rightout = dr;
        }
        else
        {
            if (dl > 0x7fff)
                dl = 0x7fff;
            else if (dl < -0x8000)
                dl = -0x8000;
            sdl = dl ^ 0xfff8000;

            if (dr > 0x7fff)
                dr = 0x7fff;
            else if (dr < -0x8000)
                dr = -0x8000;
            sdr = dr ^ 0xfff8000;

            *bothout++ = (((sdr + sdl) / 2) >> 8);
        }

        // Increment current pointers in mixbuffer.
        leftout += step;
        rightout += step;
    }

    if (updated)
    {
        // Debug check.
        if ( flag )
        {
          misses += flag;
          flag = 0;
        }

        if ( misses > 10 )
        {
          fprintf( stderr, "I_SoundUpdate: missed 10 buffer writes\n");
          misses = 0;
        }
    
        // Increment flag for update.
        flag++;
	sGameAudio->ProcessSoundSamples(mixbuffer);
//	RDebug::Print(_L("ProcessSoundSamples, count=%d"), epoc_count);
    }
}


// 
// This is used to write out the mixbuffer
//  during each game loop update.
//
void
I_SubmitSound(void)
{
	if (flag)
	{
		sGameAudio->SoundUpdate();
		flag = 0;
	}
}



void
I_UpdateSoundParams
( int   handle,
  int   vol,
  int   sep,
  int   pitch)
{
  // I fail too see that this is used.
  // Would be using the handle to identify
  //  on which channel the sound might be active,
  //  and resetting the channel parameters.

  // UNUSED.
  handle = vol = sep = pitch = 0;
}




void I_ShutdownSound(void)
{    
  // Wait till all pending sounds are finished.
  int done = 0;
  int i;
  
  if (nosound)
    return;
 
  while ( !done )
  {
    for( i=0 ; i<cv_numChannels.value && !channels[i] ; i++)
      ;
    if (i==cv_numChannels.value)
      done++;
    else
    {
        I_UpdateSound();
        I_SubmitSound();
    }
  }

  // Cleaning up -releasing the DSP device.
//  close ( audio_fd );

  // Done.
  return;
}






void
I_StartupSound()
{ 
  int i, j;

  if (nosound)
    return;

  // Secure and configure sound device first.
  fprintf( stderr, "I_InitSound: ");
  
	sGameAudio = CGameAudio::NewL(audio_8bit_flag ? (TBool) EFalse : (TBool) ETrue, SAMPLECOUNT);

  // Initialize external data (all sounds) at start, keep static.
  fprintf( stderr, "I_InitSound: ");

  // Do we have a sound lump for the chaingun?
  if (W_CheckNumForName("dschgun") == -1)
  {
    // No, so link it to the pistol sound
    //S_sfx[sfx_chgun].link = &S_sfx[sfx_pistol];
    //S_sfx[sfx_chgun].pitch = 150;
    //S_sfx[sfx_chgun].volume = 0;
    //S_sfx[sfx_chgun].data = 0;
    fprintf(stderr, "linking chaingun sound to pistol sound,");
  }
  else
  {
    fprintf(stderr, "found chaingun sound,");
  }
  
  //for (i=1 ; i<NUMSFX ; i++)
  //{ 
  //  lengths[i] = 0;
  //}


  fprintf( stderr, " pre-cached all sound data\n");
  
  // Now initialize mixbuffer with zero.
  for ( i = 0; i< MIXBUFFERSIZE; i++ )
    mixbuffer[i] = 0;
  
  // Finished initialization.
  fprintf(stderr, "I_InitSound: sound module ready\n");
}

