/*******************************************************************
 *
 *	File:		EDoom.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

extern "C"
{
#include "doomdef.h"
#include "i_system.h"
#include "m_argv.h"
#include "v_video.h"
#include "st_stuff.h"
#include "i_video.h"
#include "keys.h"

extern void D_PostEvent (const event_t* ev);

/*********************************************************
 *
 * Global variables we are supposed to declare
 *
 *********************************************************/

// this is the CURRENT rendermode!! very important : used by w_wad, and many other code
rendermode_t    rendermode = render_soft;
boolean         highcolor = false;
byte			graphics_started = 0;

void I_UpdateNoBlit (void){}	// not needed
void I_StartFrame (void){}		// not needed

void I_ShutdownGraphics(void);
void I_FinishUpdate(void);
int VID_SetMode (int modenum);
void I_StartupGraphics(void);
void I_SetPalette (byte* palette);
void I_ReadScreen (byte* scr);
char *VID_GetModeName (int modenum);
int VID_NumModes(void);
int VID_GetModeForSize( int w, int h);
}		// extern "C"



/****************************************************
 *
 * IMPLEMENTATION OF KEYBOARD HANDLER
 *
 ****************************************************/

#include "video.h"

class CKeyboardHandler : public CBase, public MKeyboardObserver
{
public:
	~CKeyboardHandler(){};
	static CKeyboardHandler* NewL();

protected:
	CKeyboardHandler(){};
	void ConstructL();

	TInt MapScanCode(TInt aScancode);
	void PostEvent(TInt aScancode, TBool aUpEvent);

	virtual void KeyDownEvent(TInt aScancode);
	virtual void KeyUpEvent(TInt aScancode);

	static const TInt sKeyMapTable[][2];
};

const TInt CKeyboardHandler::sKeyMapTable[][2] = 
{
	{EStdKeyLeftArrow, KEY_LEFTARROW},
	{EStdKeyRightArrow, KEY_RIGHTARROW},
	{EStdKeyUpArrow, KEY_UPARROW},
	{EStdKeyDownArrow, KEY_DOWNARROW},

	{EStdKeyEscape, KEY_ESCAPE},
	{EStdKeyEnter, KEY_ENTER},
	{EStdKeySpace, KEY_SPACE},

	{EStdKeyComma, ','},
	{EStdKeyFullStop, '.'},

	{EStdKeyLeftShift, KEY_SHIFT},
	{EStdKeyRightShift, KEY_SHIFT},
	{EStdKeyLeftCtrl, KEY_CTRL},
	{EStdKeyRightCtrl, KEY_CTRL},

	{EStdKeyTab, KEY_TAB},
	{EStdKeyBackspace, KEY_BACKSPACE},
	{EStdKeyDelete, KEY_DEL},

	{EStdKeyMinus, KEY_MINUS},
	{EStdKeyEquals, KEY_EQUALS},
	{EStdKeyForwardSlash, '\\'},
	{EStdKeyBackSlash, '/'},
	{EStdKeySemiColon, ';'},

	{0, 0}
};

CKeyboardHandler* CKeyboardHandler::NewL()
{
	CKeyboardHandler*	self = new (ELeave) CKeyboardHandler;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();		// self
	return self;
}

void CKeyboardHandler::ConstructL()
{
}

TInt CKeyboardHandler::MapScanCode(TInt aScancode)
{
	int i;

//	if ((aScancode>='0' && aScancode<='9') || (aScancode>='a' && aScancode<='z') || (aScancode>='A' && aScancode<='Z'))
	if ((aScancode>=' ' && aScancode<='Z'))
		return tolower(aScancode);
	for (i = 0; sKeyMapTable[i][0]; i++)
		if (sKeyMapTable[i][0] == aScancode)
			return sKeyMapTable[i][1];
	return 0;
}

void CKeyboardHandler::KeyDownEvent(TInt aScancode)
{
	PostEvent(MapScanCode(aScancode), EFalse);
}

void CKeyboardHandler::KeyUpEvent(TInt aScancode)
{
	PostEvent(MapScanCode(aScancode), ETrue);
}


void CKeyboardHandler::PostEvent(TInt aScancode, TBool aUpEvent)
{
	if (aScancode)
	{
		event_t ev;
		ev.type = aUpEvent ? ev_keyup : ev_keydown;
		ev.data1 = aScancode;
		ev.data2 = 0;
		ev.data3 = 0;
		D_PostEvent(&ev);
	}
}


/****************************************************
 *
 * IMPLEMENTATION OF VIDEO ROUTINES
 *
 ****************************************************/

static char					sColorMap[256];
static CGameWindow*			sGameWindow = NULL;
static CKeyboardHandler*	sKeyboardHandler = NULL;
static unsigned char*		sFrameBuffer = NULL;

#define MAXWINMODES 2
static char vidModeName[MAXWINMODES][32];
static int windowedModes[MAXWINMODES][2] = 
{
	{320, 200},
	{640, 400}
};


int VID_NumModes(void)
{
    return MAXWINMODES;
}

void I_StartupGraphics(void)
{
	if( graphics_started )
		return;

	VID_SetMode(0);

	graphics_started = true;
}

void I_ReadScreen (byte* scr)
{
	memcpy(scr, vid.buffer, vid.width*vid.height*vid.bpp);
}

void I_ShutdownGraphics(void)
{
    printf( "I_ShutdownGraphics\n");

    if( !graphics_started )
        return;

    graphics_started = false;
}

char *VID_GetModeName (int modenum)
{
   sprintf( vidModeName[modenum], "%dx%d", 
            windowedModes[modenum][0],
            windowedModes[modenum][1]);
   return vidModeName[modenum];
}


int VID_GetModeForSize( int w, int h)
{
    int i;

    for (i=0; i<MAXWINMODES;i++)
        if(windowedModes[i][0]==w && windowedModes[i][1]==h)
            return i;
    return 0;
}



static void BlitImage()
{
	register unsigned char*		dst = sFrameBuffer;
	register unsigned char*		frameBufferLine;
	register unsigned char*		src = vid.buffer;

	frameBufferLine = sFrameBuffer;
	for(int i=0 ; i<vid.height ; i++)
	{
		dst = frameBufferLine;
		for(int j=0 ; j<vid.width ; j++)
		{
			*dst++ = sColorMap[*src++];
		}
		frameBufferLine += vid.rowbytes;
	}
}

void I_FinishUpdate(void)
{
#ifdef __WINS__
	static int blit_count = 0;
	if (++blit_count == 3)
	{
		BlitImage();
		blit_count = 0;
	}
#else
	TBool	enable=ETrue;
	if (enable)
	{
		BlitImage();
		enable=EFalse;
	}
	else
		enable = ETrue;
#endif

#ifdef __WINS__
	sGameWindow->EmulateTargetDrawBitmap(NULL);
#endif
	sGameWindow->PollWServEvents();
	sGameWindow->GameScreen()->FrameTick();

	static count=0;
	if (++count == 60)
	{
		count = 0;
		RDebug::Print(_L("fps: %d"), sGameWindow->GameScreen()->Fps());
	}
}


static void ClearScreen()
{
	register unsigned char*		dst = sFrameBuffer;
	register unsigned char*		frameBufferLine;
	register unsigned char*		src = vid.buffer;

	frameBufferLine = sFrameBuffer;
	for(int i=0 ; i<vid.height ; i++)
	{
		dst = frameBufferLine;
		for(int j=0 ; j<vid.width ; j++)
		{
			*dst++ = 0xff;
		}
		frameBufferLine += vid.rowbytes;
	}
#ifdef __WINS__
	sGameWindow->EmulateTargetDrawBitmap(NULL);
#endif
}

static int oldMode = -1;

int VID_SetMode (int modenum)
{
	if (modenum >= MAXWINMODES) 
	{
		printf("VID_SetMode modenum >= MAXWINMODES\n");
		return -1;
	}
	if (oldMode == modenum)
		return 1;

	if (oldMode != -1)
	{
		ClearScreen();

		delete sGameWindow;
		sGameWindow = NULL;

		delete sKeyboardHandler;
		sKeyboardHandler = NULL;

		free(vid.direct);
		vid.direct = NULL;
	}
	oldMode = modenum;

	TSize	gameSize(windowedModes[modenum][0], windowedModes[modenum][1]);
	sGameWindow = CGameWindow::NewL();
	sGameWindow->CreateDisplayL(gameSize.iWidth, gameSize.iHeight, 8, EOrientationNormal, 60);

	sKeyboardHandler = CKeyboardHandler::NewL();
	sGameWindow->SetKeyboardObserver(sKeyboardHandler);

    int vidbuffersize = (gameSize.iWidth * gameSize.iHeight * NUMSCREENS)
        + (gameSize.iWidth * ST_HEIGHT);  //status bar

	vid.width  = gameSize.iWidth;
	vid.height = gameSize.iHeight;
	printf("VID_SetMode %dx%d\n", vid.width, vid.height);

	vid.direct = vid.buffer = (unsigned char*) malloc(vidbuffersize);
	vid.rowbytes = sGameWindow->GameScreen()->FrameBufferBytesPerScanLine();
	vid.bpp      = 1;
	vid.recalc   = 1;
	vid.modenum  = modenum;

	sFrameBuffer = sGameWindow->GameScreen()->FrameBufferAddress();
	if (modenum == 0)
	{
#ifdef __MARM__
		sFrameBuffer += 160 + 100*sGameWindow->GameScreen()->FrameBufferBytesPerScanLine();
#endif
		sGameWindow->GameScreen()->SetPosition(TPoint(160,100));
	}

#ifdef __WINS__
//	sGameWindow->GameScreen()->SetPosition(TPoint((screenSize.iWidth - gameSize.iWidth) >> 1, (screenSize.iHeight - gameSize.iHeight) >> 1));
	sGameWindow->GameScreen()->SetPosition(TPoint(0, 0));
#endif

	printf("VID_SetMode(%d) DONE\n", modenum);
	return 1;
}



void I_SetPalette (byte* palette)
{
	byte*  usegamma;
	usegamma = gammatable[scr_gamma%5];

	for(int i=0 ; i<256 ; i++)
	{
		int r = usegamma[*palette++];
		int g = usegamma[*palette++];
		int b = usegamma[*palette++];

		sColorMap[i] = sGameWindow->GameScreen()->FindEpocRgbColor(r, g, b);
	}
}




